@@ -1,28 +1,30 @@
 #[cfg(Py_3_8)]
 use crate::vectorcallfunc;
 use crate::{object, PyGetSetDef, PyMemberDef, PyMethodDef, PyObject, Py_ssize_t};
+use std::ffi::{c_char, c_int, c_uint, c_void};
 use std::mem;
-use std::os::raw::{c_char, c_int, c_uint, c_ulong, c_void};
 
-// skipped _Py_NewReference
-// skipped _Py_ForgetReference
-// skipped _Py_GetRefTotal
+// skipped private _Py_NewReference
+// skipped private _Py_NewReferenceNoTotal
+// skipped private _Py_ResurrectReference
 
-// skipped _Py_Identifier
+// skipped private _Py_GetGlobalRefTotal
+// skipped private _Py_GetRefTotal
+// skipped private _Py_GetLegacyRefTotal
+// skipped private _PyInterpreterState_GetRefTotal
 
-// skipped _Py_static_string_init
-// skipped _Py_static_string
-// skipped _Py_IDENTIFIER
+// skipped private _Py_Identifier
+
+// skipped private _Py_static_string_init
+// skipped private _Py_static_string
+// skipped private _Py_IDENTIFIER
 
 #[cfg(not(Py_3_11))] // moved to src/buffer.rs from Python
 mod bufferinfo {
     use crate::Py_ssize_t;
-    use std::os::raw::{c_char, c_int, c_void};
+    use std::ffi::{c_char, c_int, c_void};
     use std::ptr;
 
-    #[cfg(PyPy)]
-    const Py_MAX_NDIMS: usize = 36;
-
     #[repr(C)]
     #[derive(Copy, Clone)]
     pub struct Py_buffer {
@@ -41,12 +43,13 @@ mod bufferinfo {
         #[cfg(PyPy)]
         pub flags: c_int,
         #[cfg(PyPy)]
-        pub _strides: [Py_ssize_t; Py_MAX_NDIMS],
+        pub _strides: [Py_ssize_t; PyBUF_MAX_NDIM as usize],
         #[cfg(PyPy)]
-        pub _shape: [Py_ssize_t; Py_MAX_NDIMS],
+        pub _shape: [Py_ssize_t; PyBUF_MAX_NDIM as usize],
     }
 
     impl Py_buffer {
+        #[allow(clippy::new_without_default)]
         pub const fn new() -> Self {
             Py_buffer {
                 buf: ptr::null_mut(),
@@ -63,9 +66,9 @@ mod bufferinfo {
                 #[cfg(PyPy)]
                 flags: 0,
                 #[cfg(PyPy)]
-                _strides: [0; Py_MAX_NDIMS],
+                _strides: [0; PyBUF_MAX_NDIM as usize],
                 #[cfg(PyPy)]
-                _shape: [0; Py_MAX_NDIMS],
+                _shape: [0; PyBUF_MAX_NDIM as usize],
             }
         }
     }
@@ -79,7 +82,7 @@ mod bufferinfo {
         unsafe extern "C" fn(arg1: *mut crate::PyObject, arg2: *mut Py_buffer);
 
     /// Maximum number of dimensions
-    pub const PyBUF_MAX_NDIM: c_int = 64;
+    pub const PyBUF_MAX_NDIM: c_int = if cfg!(PyPy) { 36 } else { 64 };
 
     /* Flags for getting buffers */
     pub const PyBUF_SIMPLE: c_int = 0;
@@ -205,17 +208,8 @@ pub type printfunc =
     unsafe extern "C" fn(arg1: *mut PyObject, arg2: *mut ::libc::FILE, arg3: c_int) -> c_int;
 
 #[repr(C)]
-#[derive(Debug, Copy, Clone)]
+#[derive(Debug)]
 pub struct PyTypeObject {
-    #[cfg(all(PyPy, not(Py_3_9)))]
-    pub ob_refcnt: Py_ssize_t,
-    #[cfg(all(PyPy, not(Py_3_9)))]
-    pub ob_pypy_link: Py_ssize_t,
-    #[cfg(all(PyPy, not(Py_3_9)))]
-    pub ob_type: *mut PyTypeObject,
-    #[cfg(all(PyPy, not(Py_3_9)))]
-    pub ob_size: Py_ssize_t,
-    #[cfg(not(all(PyPy, not(Py_3_9))))]
     pub ob_base: object::PyVarObject,
     pub tp_name: *const c_char,
     pub tp_basicsize: Py_ssize_t,
@@ -238,7 +232,10 @@ pub struct PyTypeObject {
     pub tp_getattro: Option<object::getattrofunc>,
     pub tp_setattro: Option<object::setattrofunc>,
     pub tp_as_buffer: *mut PyBufferProcs,
-    pub tp_flags: c_ulong,
+    #[cfg(not(Py_GIL_DISABLED))]
+    pub tp_flags: std::ffi::c_ulong,
+    #[cfg(Py_GIL_DISABLED)]
+    pub tp_flags: crate::impl_::AtomicCULong,
     pub tp_doc: *const c_char,
     pub tp_traverse: Option<object::traverseproc>,
     pub tp_clear: Option<object::inquiry>,
@@ -274,7 +271,7 @@ pub struct PyTypeObject {
     #[cfg(any(all(PyPy, Py_3_8, not(Py_3_10)), all(not(PyPy), Py_3_8, not(Py_3_9))))]
     pub tp_print: Option<printfunc>,
     #[cfg(all(PyPy, not(Py_3_10)))]
-    pub tp_pypy_flags: std::os::raw::c_long,
+    pub tp_pypy_flags: std::ffi::c_long,
     #[cfg(py_sys_config = "COUNT_ALLOCS")]
     pub tp_allocs: Py_ssize_t,
     #[cfg(py_sys_config = "COUNT_ALLOCS")]
@@ -290,14 +287,15 @@ pub struct PyTypeObject {
 #[cfg(Py_3_11)]
 #[repr(C)]
 #[derive(Clone)]
-pub struct _specialization_cache {
-    pub getitem: *mut PyObject,
+struct _specialization_cache {
+    getitem: *mut PyObject,
     #[cfg(Py_3_12)]
-    pub getitem_version: u32,
+    getitem_version: u32,
+    #[cfg(Py_3_13)]
+    init: *mut PyObject,
 }
 
 #[repr(C)]
-#[derive(Clone)]
 pub struct PyHeapTypeObject {
     pub ht_type: PyTypeObject,
     pub as_async: PyAsyncMethods,
@@ -312,10 +310,14 @@ pub struct PyHeapTypeObject {
     pub ht_cached_keys: *mut c_void,
     #[cfg(Py_3_9)]
     pub ht_module: *mut object::PyObject,
-    #[cfg(Py_3_11)]
-    pub _ht_tpname: *mut c_char,
-    #[cfg(Py_3_11)]
-    pub _spec_cache: _specialization_cache,
+    #[cfg(all(Py_3_11, not(PyPy)))]
+    _ht_tpname: *mut c_char,
+    #[cfg(Py_3_14)]
+    pub ht_token: *mut c_void,
+    #[cfg(all(Py_3_11, not(PyPy)))]
+    _spec_cache: _specialization_cache,
+    #[cfg(all(Py_GIL_DISABLED, Py_3_14))]
+    pub unique_id: Py_ssize_t,
 }
 
 impl Default for PyHeapTypeObject {
@@ -326,20 +328,16 @@ impl Default for PyHeapTypeObject {
 }
 
 #[inline]
+#[cfg(not(Py_3_11))]
 pub unsafe fn PyHeapType_GET_MEMBERS(etype: *mut PyHeapTypeObject) -> *mut PyMemberDef {
     let py_type = object::Py_TYPE(etype as *mut object::PyObject);
     let ptr = etype.offset((*py_type).tp_basicsize);
     ptr as *mut PyMemberDef
 }
 
-// skipped _PyType_Name
-// skipped _PyType_Lookup
-// skipped _PyType_LookupId
-// skipped _PyObject_LookupSpecial
-// skipped _PyType_CalculateMetaclass
-// skipped _PyType_GetDocFromInternalDoc
-// skipped _PyType_GetTextSignatureFromInternalDoc
-// skipped _PyType_GetModuleByDef
+// skipped private _PyType_Name
+// skipped private _PyType_Lookup
+// skipped private _PyType_LookupRef
 
 extern "C" {
     #[cfg(Py_3_12)]
@@ -348,57 +346,57 @@ extern "C" {
     #[cfg_attr(PyPy, link_name = "PyPyObject_Print")]
     pub fn PyObject_Print(o: *mut PyObject, fp: *mut ::libc::FILE, flags: c_int) -> c_int;
 
-    // skipped _Py_BreakPoint
-    // skipped _PyObject_Dump
-    // skipped _PyObject_IsFreed
-    // skipped _PyObject_IsAbstract
-    // skipped _PyObject_GetAttrId
-    // skipped _PyObject_SetAttrId
-    // skipped _PyObject_LookupAttr
-    // skipped _PyObject_LookupAttrId
-    // skipped _PyObject_GetMethod
+    // skipped private _Py_BreakPoint
+    // skipped private _PyObject_Dump
 
-    #[cfg(not(PyPy))]
-    pub fn _PyObject_GetDictPtr(obj: *mut PyObject) -> *mut *mut PyObject;
-    #[cfg(not(PyPy))]
-    pub fn _PyObject_NextNotImplemented(arg1: *mut PyObject) -> *mut PyObject;
+    // skipped _PyObject_GetAttrId
+
+    // skipped private _PyObject_GetDictPtr
     pub fn PyObject_CallFinalizer(arg1: *mut PyObject);
     #[cfg_attr(PyPy, link_name = "PyPyObject_CallFinalizerFromDealloc")]
     pub fn PyObject_CallFinalizerFromDealloc(arg1: *mut PyObject) -> c_int;
 
-    // skipped _PyObject_GenericGetAttrWithDict
-    // skipped _PyObject_GenericSetAttrWithDict
-    // skipped _PyObject_FunctionStr
+    // skipped private _PyObject_GenericGetAttrWithDict
+    // skipped private _PyObject_GenericSetAttrWithDict
+    // skipped private _PyObject_FunctionStr
 }
 
 // skipped Py_SETREF
 // skipped Py_XSETREF
 
-#[cfg_attr(windows, link(name = "pythonXY"))]
-extern "C" {
-    pub static mut _PyNone_Type: PyTypeObject;
-    pub static mut _PyNotImplemented_Type: PyTypeObject;
-}
+// skipped private _PyObject_ASSERT_FROM
+// skipped private _PyObject_ASSERT_WITH_MSG
+// skipped private _PyObject_ASSERT
+// skipped private _PyObject_ASSERT_FAILED_MSG
+// skipped private _PyObject_AssertFailed
 
-// skipped _Py_SwappedOp
-
-// skipped _PyDebugAllocatorStats
-// skipped _PyObject_DebugTypeStats
-// skipped _PyObject_ASSERT_FROM
-// skipped _PyObject_ASSERT_WITH_MSG
-// skipped _PyObject_ASSERT
-// skipped _PyObject_ASSERT_FAILED_MSG
-// skipped _PyObject_AssertFailed
-// skipped _PyObject_CheckConsistency
+// skipped private _PyTrash_begin
+// skipped private _PyTrash_end
 
 // skipped _PyTrash_thread_deposit_object
 // skipped _PyTrash_thread_destroy_chain
-// skipped _PyTrash_begin
-// skipped _PyTrash_end
-// skipped _PyTrash_cond
-// skipped PyTrash_UNWIND_LEVEL
-// skipped Py_TRASHCAN_BEGIN_CONDITION
-// skipped Py_TRASHCAN_END
+
 // skipped Py_TRASHCAN_BEGIN
-// skipped Py_TRASHCAN_SAFE_BEGIN
-// skipped Py_TRASHCAN_SAFE_END
+// skipped Py_TRASHCAN_END
+
+// skipped PyObject_GetItemData
+
+// skipped PyObject_VisitManagedDict
+// skipped _PyObject_SetManagedDict
+// skipped PyObject_ClearManagedDict
+
+// skipped TYPE_MAX_WATCHERS
+
+// skipped PyType_WatchCallback
+// skipped PyType_AddWatcher
+// skipped PyType_ClearWatcher
+// skipped PyType_Watch
+// skipped PyType_Unwatch
+
+// skipped PyUnstable_Type_AssignVersionTag
+
+// skipped PyRefTracerEvent
+
+// skipped PyRefTracer
+// skipped PyRefTracer_SetTracer
+// skipped PyRefTracer_GetTracer
